/**********************************************************************************************/
/* FBL_Task.h																			 	  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/
 
#ifndef __FBL_Task_h
	#define __FBL_Task_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#	include <VSDK/FBL_String.h>
#else
#	include <FBL.h>
#	include <FBL_String.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_Progresser;
class FBL_Thread;
class FBL_ArraySet;

 
/**********************************************************************************************/
class FBL_Task 
{
	public://///////////////////////////////////////////////////////////////////////////////////
		
		enum State { Ready, Running, Finished, Canceled };

static	ulong 					sLastYieldTick;		// last tick when was made Yield
static 	ulong					sMaxTicks;			// Max number of ticks we can use for a task.
		
		ushort					mDonePercent;		// Current state of task progress if it 
													// can be measured.

		FBL_Thread*				mThread;			// Callback pointer.


	public://///////////////////////////////////////////////////////////////////////////////////
 
FBL_EXP							FBL_Task( 
									const char* 	inName ); 

FBL_EXP virtual					~FBL_Task( void );

FBL_EXP void					SetProgresser( FBL_Progresser*	inProgresser, bool inOwns );
		FBL_Progresser*			GetProgresser( void ) const { return mProgresser; }  

		void 					SetAffectedRows(
									FBL_ArraySet*	inAffectedRows,
									bool			inOwnAffectedRows )
								{
									mAffectedRows = inAffectedRows;
									mOwnAffectedRows = inOwnAffectedRows;
								} 
									
		FBL_ArraySet*			GetAffectedRows( void ) const { 
									return mAffectedRows; 
								}

								// do nothing in this class, must be overriden.
virtual	void					Run( void ) = 0;	

		State					GetState( void ) const 	{ return mState; 	}

FBL_EXP virtual void			SetState( State inState );

		long 					GetError( void ) const 	{ return mErrCode; 	}
		void 					SetError( long inErr )  { mErrCode = inErr; }

FBL_EXP	void					SetName( const char* inName );

		FBL_String				GetName( void ) const  { return mName; }

		bool					IsRunning( void ) const { return mState == Running; }
		
		bool					IsDone( void ) const { 
									return (mState == Finished || mState == Canceled); 
								}


	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		FBL_String				mName;				// Name of the task;
		State					mState;				// Current state of the task.

		FBL_ArraySet*			mAffectedRows;		// BitSet of affected rows. Optional.
		bool					mOwnAffectedRows;
		
		long					mErrCode;			// Here thread can store FBL error if any.
		
		FBL_Progresser*			mProgresser;		// something what displays progress of the task.
													// Can be NULL.	
		bool					mOwnProgresser;		// true if owns progresser.


	public://///////////////////////////////////////////////////////////////////////////////////

 		// "Callback" methods:
		// The following methods are called from the kernel,
		// you can customise them as you wish. 

FBL_EXP static	long			DoIndexing( FBL_Task* inIndexTask );

};


/**********************************************************************************************/
class StTaskLock 
{
	public://///////////////////////////////////////////////////////////////////////////////////
 
								StTaskLock( FBL_Task* inTask ) {
									mTask = inTask;
									mTask->SetState( FBL_Task::Running );
								}	

								~StTaskLock( void ) {
									mTask->SetState( FBL_Task::Finished );
								}	

	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		FBL_Task*				mTask;	
};


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif
