/**********************************************************************************************/
/* VDK_BaseObject.h																			  */
/*                                                                       					  */
/* Valentina SDK    																		  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/

#ifndef __VDK_BaseObject_h 
	#define __VDK_BaseObject_h

#if VSDK_FRAMEWORK_STYLE
#include <VSDK/FBL_String.h>
#include <VSDK/FBL_LanguageManager.h>

#include <VSDK/VDK_Link.h>
#include <VSDK/VDK_Field.h>

#include <VSDK/VDK_Boolean.h>
#include <VSDK/VDK_Byte.h>
#include <VSDK/VDK_Short.h>
#include <VSDK/VDK_UShort.h>
#include <VSDK/VDK_Medium.h>
#include <VSDK/VDK_UMedium.h>
#include <VSDK/VDK_Long.h>
#include <VSDK/VDK_ULong.h>
#include <VSDK/VDK_LLong.h>
#include <VSDK/VDK_ULLong.h>
#include <VSDK/VDK_Float.h>
#include <VSDK/VDK_Double.h>
#include <VSDK/VDK_Date.h>
#include <VSDK/VDK_Time.h>
#include <VSDK/VDK_DateTime.h>
#include <VSDK/VDK_String.h>
#include <VSDK/VDK_VarChar.h>
#include <VSDK/VDK_FixedBinary.h>
#include <VSDK/VDK_VarBinary.h>
#include <VSDK/VDK_BLOB.h>
#include <VSDK/VDK_Text.h>
#include <VSDK/VDK_Picture.h>
#include <VSDK/VDK_ObjectPtr.h>
#else
#include <FBL_String.h>
#include <FBL_LanguageManager.h>

#include <VDK_Link.h>
#include <VDK_Field.h>

#include <VDK_Boolean.h>
#include <VDK_Byte.h>
#include <VDK_Short.h>
#include <VDK_UShort.h>
#include <VDK_Medium.h>
#include <VDK_UMedium.h>
#include <VDK_Long.h>
#include <VDK_ULong.h>
#include <VDK_LLong.h>
#include <VDK_ULLong.h>
#include <VDK_Float.h>
#include <VDK_Double.h>
#include <VDK_Date.h>
#include <VDK_Time.h>
#include <VDK_DateTime.h>
#include <VDK_String.h>
#include <VDK_VarChar.h>
#include <VDK_FixedBinary.h>
#include <VDK_VarBinary.h>
#include <VDK_BLOB.h>
#include <VDK_Text.h>
#include <VDK_Picture.h>
#include <VDK_ObjectPtr.h>
#endif // VSDK_FRAMEWORK_STYLE


#if defined(__MWERKS__)
	#pragma options align=power
#endif

/**********************************************************************************************/
enum VDK_InheritanceType
{
	kVDK_RootObject,			// Object has no parent
	kVDK_HardInheritance,		// Object was made with hard inheritance
	kVDK_SoftInheritance		// Object was made with soft inheritance
};

enum VDK_ChildsType
{
	kVDK_NoChilds,
	kVDK_HardChilds,
	kVDK_SoftChilds
};



/**********************************************************************************************/
// internal classes:
class FBL_BaseField;
class OBL_BaseObject; 

// opened classes:
class FBL_Set;
class FBL_BitSet;
class FBL_ArraySet;
class FBL_Task;

// VDK classes
class VDK_BaseObject;
class VDK_DataBase;
class VDK_Field;
class VDK_Link;
class VDK_SortOrder;
class VDK_SearchTree;
class VDK_ULong;

/**********************************************************************************************/
typedef FBL_Array<VDK_Field*> 		FieldsArray;
typedef FBL_Array<VDK_Link*>		LinksArray;
typedef FBL_Array<VDK_BaseObject*> 	ObjectsArray;


/**********************************************************************************************/
class FBL_EXP VDK_BaseObject  
{
		friend class VDK_DataBase;
		friend class VDK_Field;
		friend class VDK_ObjectPtr;
	
	public://///////////////////////////////////////////////////////////////////////////////////
 
FBL_EXP							VDK_BaseObject( const char*	inName );
FBL_EXP virtual 				~VDK_BaseObject( void );

		virtual	VDK_BaseObject*	Clone( void ) { return NULL; }
				
FBL_EXP virtual const char*		GetName( void ) const;
FBL_EXP virtual void 			SetName( const char* inNewName );

		VDK_DataBase*			GetDataBase( void ) const { return mDataBase; }


		/* Encryption methods: */

FBL_EXP	bool					IsEncrypted( void ) const;		
		
FBL_EXP	void					SetEncryption( 
									FBL_EncryptionKeyCPtr inKey );
									
FBL_EXP	void					ChangeEncryption( 
									FBL_EncryptionKeyCPtr inOldKey,
									FBL_EncryptionKeyCPtr inNewKey );


		/* Field methods: */		
				
								// This method don't count RecID field, it counts only custom fields.
FBL_EXP	virtual ushort			GetFieldCount( FBL_FieldKind inFieldKind = kFBL_All ) const;
		
								// Returns field by name. This method can return RecID field.
FBL_EXP	virtual	VDK_Field*		GetField( const char* inName ) const; 
							
								// Returns field by index. This method don't return RecID field. 
		virtual	VDK_Field*		GetField( ushort inIndex ) const;	  
		
								// This method returns index of the field, 
								// it takes in account that RecID field is not "custom field". 
		virtual	ulong			GetIndexOfField( VDK_Field* inField ) {
									ulong index = mFields.FindIndex( inField );
									return index ? index - 1 : 0;
								}	
								
		const FieldsArray*		GetFieldsArray( void ) const { return &mFields; }
		
								// make a copy on the heap and copy pointers on fields 
								// of specified kind only.										
FBL_EXP	FieldsArray*			GetFieldArrayCopy( FBL_FieldKind inFieldKind );
								
									
		/* Record's methods: */
		
								// Returns count of records in BaseObject,
FBL_EXP	virtual	ID_TYPE			GetRecordCount( void ) const;	
			
								// Returns the count of physical records including deleted. 
FBL_EXP	virtual	ID_TYPE			GetPhysicalRecordCount( void ) const;
		
								// Clear buffer of record in RAM.
								// Call SetBlank for each field of Table.	
FBL_EXP virtual void			SetBlank( void );	
		
								// Add to the Table new records with current value
								// of fields.
FBL_EXP virtual FBL_REC_ID		AddRecord( void );

								// Update current record of Table with current values
								// of fields. Update also indexes if any.
FBL_EXP virtual void			UpdateRecord( void );

								// Delete current record. If exists next logical record, 
								// go to it, else go to the previouse record. 
								// If Table become empty then current record undefined.
								// Returns false if record can't be deleted.
FBL_EXP virtual bool			DeleteRecord( void );

								// Delete all records in BaseObject.
								// Table become empty, current record undefined.
FBL_EXP virtual	void			DeleteAllRecords( void );

								// Delete records from selection.
								// If Table become empty, current record undefined.
FBL_EXP virtual	void			DeleteRecords( FBL_Set* inSet );


		/* Navigation methods: */
		
								// Go to the first record.
								// Read record from disk to the memory buffer.
FBL_EXP virtual bool			FirstRecord( void );	

								// Go to the last record.
								// Read record from disk to the memory buffer.
FBL_EXP virtual bool			LastRecord( void );

								// Returns true if next record found.
								// False if current record is last in BaseObject. 
								// Read record from disk to the memory buffer.
FBL_EXP virtual bool			NextRecord( void );
			
								// Returns true if prev record found.
								// False if current record is first in BaseObject. 
								// Read record from disk to the memory buffer.
FBL_EXP virtual bool			PrevRecord( void );

								// Returns RecID of current record, range is 1..N 
								// 0 - if record is undefined
FBL_EXP virtual FBL_REC_ID		GetRecID( void ) const;

								// Make record with specified inRecID current.
								// Read record from disk to the memory buffer.
								// Returns true if everything Ok.
FBL_EXP virtual bool			GoToRecord( FBL_REC_ID inRecID );
					
								// Returns true if a record with inRecID exists in BaseObject.
FBL_EXP	virtual	bool 			Exists( FBL_REC_ID inRecID );
							
								
		/* Record locking: */		
		
FBL_EXP virtual	bool			LockRecord	( 
									FBL_REC_ID 	inRecID, 
									ELockType	inLockType,
									ulong		inOwnerID );
									
FBL_EXP virtual	void			UnlockRecord( 
									FBL_REC_ID 	inRecID,
									ulong		inOwnerID );
					
								
		/* Selection methods: */
		
								// Returns selection of all records in BaseObject.
FBL_EXP virtual FBL_BitSet*		SelectAllRecords( void );
								
								// Returns empty selection.								
FBL_EXP virtual FBL_BitSet*		SelectNone( void );
		
								// Perform compound search by several fields of the BaseObject
FBL_EXP virtual	FBL_BitSet* 	MakeSearch( VDK_SearchTree* inSearchTree );

								// Sort Set of records by specified field.
								// Returns always FBL_ArraySet. 
FBL_EXP virtual FBL_ArraySet* 	Sort( 	FBL_Set*		inSet,
										VDK_Field& 		inSortField,
										bool			inAscending = true ); 
		
								// Sort Set of records by specified fields.
								// Returns always FBL_ArraySet. 
FBL_EXP virtual FBL_ArraySet* 	Sort( 	FBL_Set*		inSet,
										VDK_SortOrder* 	inSortOrder ); 
		
		
		/* Cache methods: */
		
								// Flush on disk all not saved information from this 
								// BaseObject if any.
FBL_EXP virtual void			Flush( void );
		

		/* Methods to change Table structure dynamically */

				// Adds to this BaseObject(Table) a new field.
				// String, VarChar: inParam1 is max len, 			inParam2 is (long)(FBL_Language*)pLanguage
				// BLOB: 			inParam1 is segment size, 		inParam2 is nothing.
				// Text: 			inParam1 is segment size, 		inParam2 is (long)(FBL_Language*)pLanguage
				// ObjectPtr: 		inParam1 is Target BaseObject, 	inParam2 is Deletion Control. 
FBL_EXP virtual VDK_Field*		MakeNewField( 
									const char* inName, 
									uchar 		inType, 
									long		inParam1 = 0,	
									long 		inParam2 = 0,
									const char* inMethod = NULL );
		
								// Remove field from the Table.
								// This function change structure of Table!
FBL_EXP	virtual	void			DeleteField( VDK_Field* inField );

								// Change the type of the field. This function changes structure of Table!
								// inParam is like Param1 in MakeNewField()
FBL_EXP	virtual VDK_Field*		ChangeFieldType( VDK_Field* inField, uchar inNewType, ulong inParam = 0 );	


		/* Set of explicite shortcuts to create fields 	*/
		/* They are just wrappers to MakeNewField() 	*/

FBL_EXP virtual VDK_Boolean*	MakeBooleanField( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_Byte*		MakeByteField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_Short*		MakeShortField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_UShort*		MakeUShortField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_Medium*		MakeMediumField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_UMedium*	MakeUMediumField( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_Long*		MakeLongField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_ULong*		MakeULongField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_LLong*		MakeLLongField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_ULLong*		MakeULLongField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_Float*		MakeFloatField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_Double*		MakeDoubleField	( const char* inName, const char* inMethod = NULL );

FBL_EXP virtual VDK_Date*		MakeDateField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_Time*		MakeTimeField	( const char* inName, const char* inMethod = NULL );
FBL_EXP virtual VDK_DateTime*	MakeDateTimeField( const char* inName, const char* inMethod = NULL );

FBL_EXP virtual VDK_String*		MakeStringField	( const char* inName, ushort inMaxLength, 
												  const char* inLanguage = "ASCII", const char* inMethod = NULL );
												 
FBL_EXP virtual VDK_VarChar*	MakeVarCharField( const char* inName, ushort inMaxLength, 
												  const char* inLanguage = "ASCII", const char* inMethod = NULL );

FBL_EXP virtual VDK_FixedBinary*MakeFixedBinaryField( const char* inName, ushort inMaxLength, const char* inMethod = NULL );
FBL_EXP virtual VDK_VarBinary*	MakeVarBinaryField( const char* inName, ushort inMaxLength, const char* inMethod = NULL );

FBL_EXP virtual VDK_BLOB*		MakeBLOBField	( const char* inName, ulong inSegmentSize = 1024 );
FBL_EXP virtual VDK_Text*		MakeTextField	( const char* inName, ulong inSegmentSize = 1024, 
												  const char* inLanguage = "ASCII", const char* inMethod = NULL );
FBL_EXP virtual VDK_Picture*	MakePictureField( const char* inName, ulong inSegmentSize = 1024 );

FBL_EXP virtual VDK_ObjectPtr*	MakeObjectPtr( const char* inName, VDK_BaseObject* inTarget, FBL_DeletionControl inDC );


		/* Database Inheritance Methods: */ 

		bool					GetAbstract( void ) const			{ return mAbstract; 				}
		void					SetAbstract( bool inAbstract ) 		{ mAbstract = inAbstract; 			}

		VDK_ChildsType 			GetChildsType( void )				{ return mChildsType;				}
		VDK_InheritanceType		GetInheritanceType( void ) const	{ return mInheritanceType; 			}

		VDK_BaseObject*			GetChild( ushort i )				{ return mChilds.GetItemAt(i);		}
		ushort					GetChildsNumber( void )				{ return (ushort)mChilds.GetCount();}
		bool					HasAbstractChild( void );

		VDK_BaseObject*			GetParent( void )	 { return mParent; }
		VDK_BaseObject* 		GetRootObject( void ){ return mParent ? mParent->GetRootObject() : this; }


		/* Link Methods: */
			
		void					AddLink		( VDK_Link& inLink ) { mLinks.AddItem( &inLink ); 		}
		void 					RemoveLink	( VDK_Link& inLink ) { mLinks.RemoveItem( &inLink );	}
		
		ushort					GetLinksCount( void ) const	{ return (ushort) mLinks.GetCount();	}
		
		VDK_Link*				GetLink( ushort Index ) 	{ return mLinks.GetItemAt(Index);	}
FBL_EXP	VDK_Link*				GetLink( const char* inLinkName );
		
								// Returns count of links with given object. (0..) 
FBL_EXP	ushort					GetLinksCountWith( VDK_BaseObject* inBaseObject );

								// Returns link of this BaseObject with pBaseObject.
								// If there is several links can be used index
								// If there is no such link returns NULL.
FBL_EXP	VDK_Link*				GetLinkWith( VDK_BaseObject* inBaseObject, ushort index = 0 );


	public:///////////////////////////////////////////////////////////////////////////////

		// internal methods:
FBL_EXP	void					Create_OBL_Fields( void );
FBL_EXP	void					Create_VDK_Fields( void );
		
								// Create VDK field using information of the OBL field
FBL_EXP	VDK_Field*				Create_VDK_Field( FBL_BaseField* inField );
					
								// Clone VDK field.
FBL_EXP	VDK_Field* 				Clone_VDK_Field( VDK_Field* inField );

FBL_EXP	VDK_Field* 				Create_VDK_Field_aux( FBL_BaseField* inField, bool inCloned = false );
							
								// Creates instance of VDK-field					
FBL_EXP virtual	VDK_Field*		NewVDKField( const char* inName, uchar inType );
					
		bool					IsInited( void ) const 			{ return (mObjectImp != NULL); 	}
		OBL_BaseObject*			GetImplementation( void ) const { return mObjectImp;  			}

								// Returns true if at least one existing VDK_Database has a pointer
								// on a specified inBaseObject. Can be used for debugging.
FBL_EXP static	bool			Exists( VDK_BaseObject* inBaseObject );
                               
                               
	public:///////////////////////////////////////////////////////////////////////////////

FBL_EXP static VDK_BaseObject*	sBaseObject;		// used to connect fields to the BaseObject.


	protected:///////////////////////////////////////////////////////////////////////////////

		FBL_String				mName;				// up to 255 characters.
		FieldsArray				mFields;	
		VDK_DataBase*			mDataBase;			// DataBase to which belons this BaseObject.

		VDK_ULong*				mRecID;				// Each BaseObject has ID field with name "RecID".
		
		OBL_BaseObject*			mObjectImp;			// Pointer to kernel's object
		
    		/* Data members implementing database inheritance */

 		bool					mAbstract;			// True if the BaseObject is abstract.
 	
 	  	VDK_BaseObject*			mParent;			// Pointer on parent BaseObject if any.
     	VDK_InheritanceType		mInheritanceType;	// Type of inhertinace for this BaseObject.

 		ObjectsArray			mChilds;			// Array of BaseObject - direct childs. 
		VDK_ChildsType			mChildsType;		// Type of its childs: hard or soft.		
		
    		/* Data members implementing links */

		LinksArray				mLinks;				// Array of links of this BaseObject.


	public://///////////////////////////////////////////////////////////////////////////////////

 		// These 2 data members allows easy to use VSDK to develop plugins
		// such as Valentina for REALbasic, Valentina for Java, which do have garbage collection.
			
			// An owner should set up this back reference, so when destructor of this class
			// will be called FROM kernel it will set back ref to NULL via mBackRef. 
			// The owner will be able to handle correctly this situation and won't try access
			// already destroyed kernel object.
		VDK_BaseObject**		mBackRef;
				
			// here you can put pointer on your own additional data (or even structure)
			// if you need attach several variables to this C++ instance.	
		void*					mXtraData;	

			// Destructor ~VDK_BaseObject() will call this pointer if you set it.
		void					(*mpFinalize)(VDK_BaseObject* inBaseObject );
}; 


/**********************************************************************************************/
inline VDK_Field* VDK_BaseObject::GetField( ushort inIndex ) const
{
	if( 1 <= inIndex && inIndex < mFields.GetCount() )   
			return mFields.GetItemAt( ulong(inIndex + 1) ); // +1 to skip RecID field. 
	else	return NULL;
}


/**********************************************************************************************/
inline VDK_Boolean* VDK_BaseObject::MakeBooleanField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Boolean*>( MakeNewField( inName, kFBL_TypeBoolean, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Byte* VDK_BaseObject::MakeByteField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Byte*> (MakeNewField( inName, kFBL_TypeByte, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Short* VDK_BaseObject::MakeShortField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Short*> (MakeNewField( inName, kFBL_TypeShort, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_UShort* VDK_BaseObject::MakeUShortField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_UShort*> (MakeNewField( inName, kFBL_TypeUShort, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Medium* VDK_BaseObject::MakeMediumField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Medium*> (MakeNewField( inName, kFBL_TypeMedium, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_UMedium* VDK_BaseObject::MakeUMediumField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_UMedium*> (MakeNewField( inName, kFBL_TypeUMedium, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Long* VDK_BaseObject::MakeLongField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Long*> (MakeNewField( inName, kFBL_TypeLong, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_ULong* VDK_BaseObject::MakeULongField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_ULong*> (MakeNewField( inName, kFBL_TypeULong, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_LLong* VDK_BaseObject::MakeLLongField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_LLong*> (MakeNewField( inName, kFBL_TypeLLong, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_ULLong* VDK_BaseObject::MakeULLongField(
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_ULLong*> (MakeNewField( inName, kFBL_TypeULLong, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Float* VDK_BaseObject::MakeFloatField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Float*> (MakeNewField( inName, kFBL_TypeFloat, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Double* VDK_BaseObject::MakeDoubleField(
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Double*> (MakeNewField( inName, kFBL_TypeDouble, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Date* VDK_BaseObject::MakeDateField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Date*> (MakeNewField( inName, kFBL_TypeDate, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_Time* VDK_BaseObject::MakeTimeField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_Time*> (MakeNewField( inName, kFBL_TypeTime, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_DateTime* VDK_BaseObject::MakeDateTimeField( 
	const char* inName, const char* inMethod )
{
	return dynamic_cast<VDK_DateTime*> (MakeNewField( inName, kFBL_TypeDateTime, 0, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_String* VDK_BaseObject::MakeStringField( 
	const char* inName, ushort inMaxLength, 
	const char* inLanguage, const char* inMethod )							 
{
	FBL_Language* pLanguage = FBL_LanguageManager::GetLanguage( inLanguage );
	return dynamic_cast<VDK_String*> (MakeNewField( inName, kFBL_TypeString, (long)inMaxLength, (long)pLanguage, inMethod ));
}

/**********************************************************************************************/
inline VDK_VarChar* VDK_BaseObject::MakeVarCharField( 
	const char* inName, ushort inMaxLength, 
	const char* inLanguage, const char* inMethod )
{
	FBL_Language* pLanguage = FBL_LanguageManager::GetLanguage( inLanguage );
	return dynamic_cast<VDK_VarChar*> (MakeNewField( inName, kFBL_TypeVarChar, (long)inMaxLength, (long)pLanguage, inMethod ));
}

/**********************************************************************************************/
inline VDK_FixedBinary* VDK_BaseObject::MakeFixedBinaryField( 
	const char* inName, ushort inMaxLength, const char* inMethod )
{
	return dynamic_cast<VDK_FixedBinary*> (MakeNewField( inName, kFBL_TypeFixedBinary, (long)inMaxLength, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_VarBinary* VDK_BaseObject::MakeVarBinaryField( 
	const char* inName, ushort inMaxLength, const char* inMethod )
{
	return dynamic_cast<VDK_VarBinary*> (MakeNewField( inName, kFBL_TypeVarBinary, (long)inMaxLength, 0, inMethod ));
}

/**********************************************************************************************/
inline VDK_BLOB* VDK_BaseObject::MakeBLOBField	( 
	const char* inName, ulong inSegmentSize )
{
	return dynamic_cast<VDK_BLOB*> (MakeNewField( inName, kFBL_TypeBLOB, (long)inSegmentSize, 0, NULL ));
}

/**********************************************************************************************/
inline VDK_Text* VDK_BaseObject::MakeTextField	( 
	const char* inName, ulong inSegmentSize, 
	const char* inLanguage, const char* inMethod )
{
	FBL_Language* pLanguage = FBL_LanguageManager::GetLanguage( inLanguage );
	return dynamic_cast<VDK_Text*> (MakeNewField( inName, kFBL_TypeText, (long)inSegmentSize, (long)pLanguage, inMethod ));
}

/**********************************************************************************************/
inline VDK_Picture* VDK_BaseObject::MakePictureField( 
	const char* inName, ulong inSegmentSize )
{
	return dynamic_cast<VDK_Picture*> (MakeNewField( inName, kFBL_TypePicture, (long)inSegmentSize, 0, NULL ));
}

/**********************************************************************************************/
inline VDK_ObjectPtr* VDK_BaseObject::MakeObjectPtr( 
	const char* inName, VDK_BaseObject* inTarget, FBL_DeletionControl inDC )
{
	return dynamic_cast<VDK_ObjectPtr*> (MakeNewField( inName, kFBL_TypeObjectPtr, (long)inTarget, (long)inDC, NULL ));
}


/**********************************************************************************************/
#endif  

