/**********************************************************************************************/
/* VDK_DataBase.h																			  */
/*                                                                    		 				  */
/* Valentina SDK														  					  */
/*                                                                    		 				  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                		  				  */
/**********************************************************************************************/

#ifndef __VDK_DataBase_h
	#define __VDK_DataBase_h

#if VSDK_FRAMEWORK_STYLE
#include <VSDK/OBL_SQL.h>
#include <VSDK/FBL_String.h>
#include <VSDK/FBL_ToFromString.h>
#include <VSDK/OBL_Dump.h>
#include <VSDK/VDK_BaseObject.h>
#include <VSDK/VDK_Link.h>
#include <VSDK/VDK_Cursor.h>
#include <VSDK/VDK_Parameter.h>
#else
#include <OBL_SQL.h>
#include <FBL_String.h>
#include <FBL_ToFromString.h>
#include <OBL_Dump.h>
#include <VDK_BaseObject.h>
#include <VDK_Link.h>
#include <VDK_Cursor.h>
#include <VDK_Parameter.h>
#endif // VSDK_FRAMEWORK_STYLE


#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
// Windows.h have GetDateFormat macro. Ops. 
// So we undef that macro for this header.
#undef GetDateFormat


/**********************************************************************************************/
// internal classes:
class OBL_BaseObject;
class OBL_DataBase; 

// opened classes:
class FBL_BitSet;  

// VDK classes:
class VDK_Field;
class VDK_DataBase;
class VDK_Cursor;

class VDK_DescriptionFile;

struct fieldInfo;


/**********************************************************************************************/
typedef FBL_Array<VDK_DataBase*> 	ArrayOfDataBase;
typedef FBL_Array<VDK_BaseObject*> 	ObjectsArray;
typedef FBL_Array<VDK_Link*>		LinksArray;
typedef FBL_Array<VDK_Cursor*>		CursorsArray;


/**********************************************************************************************/
// This factory can be used if we work with VDK.dll to force creation of VDK_DataBase
// object on the correct heap.
//
FBL_EXP VDK_DataBase* CreateDataBase( void );


/**********************************************************************************************/
class FBL_EXP VDK_DataBase 
{
		friend class VDK_BaseObject;

	public:///////////////////////////////////////////////////////////////////////////////

FBL_EXP static VDK_DataBase*	sDataBase;		// Current database.
FBL_EXP static ArrayOfDataBase*	sDBList;		// Array of all databases that kernel know about.


static	bool					Exists( VDK_DataBase* inDataBase ){
									if( sDBList && sDBList->Include(inDataBase) )
											return true;
									else	return false;
								}
		
		
	public://///////////////////////////////////////////////////////////////////////////////////


	// This 2 functions will become absolute, 
	// so please replace call to this functions in your code on ValentinaInit/ShutDown(). 
FBL_EXP static	void			InitStaticMembers( void );
FBL_EXP static	void			DestroyStaticMembers( void );

 
 		/* Construction/destruction: */
 
FBL_EXP 						VDK_DataBase( void );								
FBL_EXP virtual					~VDK_DataBase( void );
				

		/* Disk file methods: */		
				
		// -----------------
		// CREATE:		
							// Creates on the disk a new database at the specified location.
							// First function use full path. It works on all platforms.
							// 	but for each platform must be specified correct path:
							//  	Win:		Volume\File
							//		Mac: 		Volume:Fille
							// 		Unix/Macho:	Volume/File
							// And for MacOS provided 2 native calls with FSSpec and FSRef
							// 
FBL_EXP virtual	void		CreateDataBase( 
								const char*		inFullPath,						// location
								ulong			inSizeOfCluster = (32 * 1024L),	// segment of database files.
								FBL_DbMode		inMode = kFBL_Dsc_Dat_Blb_Ind,	// number of db files.
								uchar			inNativeOS = 0 ); 				// format of database files.

#if FBL_MAC 
FBL_EXP virtual	void		CreateDataBase( 
								const FSSpec* 	inFileSpec,						// location
								ulong			inSizeOfCluster = (32 * 1024L),	// segment of database files.
								FBL_DbMode		inMode = kFBL_Dsc_Dat_Blb_Ind,	// number of db files.
								uchar			inNativeOS = 0 ); 				// format of database files.
#endif // FBL_MAC


		// -----------------
		// OPEN:		
								// Opens an exisiting database at the specified location.
FBL_EXP virtual void		OpenDataBase( 	
								const char*		inFileSpec, 
								const char* 	inPassword = NULL,
								bool 			inCheckIndexes = true ); 
		
#if FBL_MAC 
FBL_EXP virtual void		OpenDataBase( 	
								const FSSpec* 	inFileSpec, 
								const char* 	inPassword = NULL,
								bool 		 	inCheckIndexes = true ); 
#endif // FBL_MAC


		// -----------------

FBL_EXP virtual bool			IsOpen( void ) const; 
		
FBL_EXP virtual	void			CloseDataBase( void );
										
FBL_EXP	virtual	ushort			GetSchemaVersion( void ) const;
FBL_EXP	virtual	void			SetSchemaVersion( ushort inVersion );
	
FBL_EXP	virtual	ulong			GetMode( void ) const;
FBL_EXP	virtual ulong			GetSegmentSize( void ) const;

	
	public:///////////////////////////////////////////////////////////////////////////////
			
		// SQL methods:

FBL_EXP	virtual VDK_Cursor*		SqlSelect( 
									const char* 		inSqlCommand,
									ECursorLocation		inLocation = kClient, 
									ELockType 			inType = kReadOnly,
									ECursorDirection 	inDirection = kForwardOnly );

								// Execute any SQL command that not return cursor as result,
								// i.e. any command except SELECT.
FBL_EXP	virtual	void			SqlExecute( 
									const char* 		inSqlCommand,
									ulong* 				outAffectedRows = NULL,
									BindParameters* 	inBindParameters = NULL );
								

	public:///////////////////////////////////////////////////////////////////////////////
			
		/* Password methods: */
		
FBL_EXP virtual	bool			SetPassword( const char* inOldPassword, const char* inNewPassword );
		
		
		/* Encryption methods: */
		
FBL_EXP bool					IsEncrypted( void ) const;	

FBL_EXP void					SetEncryption( 
									FBL_EncryptionKeyCPtr inKey );
							
FBL_EXP void					ChangeEncryption( 
									FBL_EncryptionKeyCPtr inOldKey,
									FBL_EncryptionKeyCPtr inNewKey );

FBL_EXP void					SetStructureEncryption( 
									FBL_EncryptionKeyCPtr inKey );
							
FBL_EXP void					ChangeStructureEncryption( 
									FBL_EncryptionKeyCPtr inOldKey,
									FBL_EncryptionKeyCPtr inNewKey );
		
FBL_EXP	bool 					IsValidKey( 
									FBL_EncryptionKeyCPtr inKey );
		
		/* Accessories: */
		
		const FBL_String&		GetName( void ) const { return mName; }
FBL_EXP	const FBL_String& 		GetFullPath( void ) const;

								// Accept as C-strings so Pascal strings, because they wil
								// be converted to FBL_String, case insensitive.
								// Assume that there is no 2 dbs with the same name.
		static VDK_DataBase*	GetDataBase( FBL_String inName );

								// This one is case sensetive and accept only C strings.
FBL_EXP static VDK_DataBase*	GetDataBase( const char* inName );	

								// Do the same, just use the full path as an unique name.
FBL_EXP	static VDK_DataBase*	GetDataBaseByPath( const char* inFullPath );		
	
		static ArrayOfDataBase* GetDataBaseList( void ) { return sDBList; }
		
FBL_EXP virtual	bool			IsReadOnly( void ) const;

		
		/* BaseObject methods: */		
				
								// Returns count of BaseObjects in application domain.
FBL_EXP virtual ulong			GetBaseObjectCount( void );
								
								// Returns BaseObject by index.
		VDK_BaseObject*			GetBaseObject( ulong inIndex ) const; 
								
								// Returns BaseObject by name, case insensitive.	
FBL_EXP	VDK_BaseObject*			GetBaseObject( const char* inName  ) const; 
										
											
		/* Cache methods: */
		
								// Flush on disk all information of this database. 
FBL_EXP virtual void			Flush( void );


		/* Methods to change the database structure */
		/* only for DBMS-like applications !!!		*/
		 
								// Make in the database a new empty BaseObject(Table)
								// At least one field must be added to it.
FBL_EXP virtual VDK_BaseObject*	MakeNewBaseObject( const char* inName );

								// Deletes from database BaseObject, 
								// deletes all its information from the disk.
FBL_EXP virtual void			DeleteBaseObject( VDK_BaseObject* inBaseObject );

								// Returns the description file with corresponded number
								// for this database. If it doesn't exists on disk, 
								// creates it if inAutoCreate
								// This file is stored in the "database.vdb" file
FBL_EXP	VDK_DescriptionFile* 	GetDescriptionFile( ushort inNumber, bool inAutoCreate = true );

		
		// ---------------------
		// Dump methods:
		
								// Dump an exisiting database at the specified location.
FBL_EXP virtual void			Dump( 
									const char* 	inFilePath, 
									OBL_Dump::Type 	inType); 

								// Create new fresh Database at location inNewDBPath using DUMP.
FBL_EXP virtual void			LoadDump( 
									const char* 	inDumpFilePath, 
									const char* 	inNewDBPath, 
									OBL_Dump::Type 	inType); 
#ifdef FBL_MAC
FBL_EXP virtual void			Dump( 
									const FSSpec* 	inFileSpec, 
									OBL_Dump::Type 	inType); 

FBL_EXP virtual void			LoadDump( 
									const FSSpec* 	inDumpFileSpec, 
									const FSSpec* 	inNewDBSpec, 
									OBL_Dump::Type 	inType); 
#endif

	public:///////////////////////////////////////////////////////////////////////////////
			
		/* Database Properties: */

FBL_EXP	virtual	void			SetTimeOut( ulong inSeconds = ulong(-1) );

			// Date Time format methods:

FBL_EXP	virtual	FBL_DTFormat*	GetDTFormat( void )	const;

FBL_EXP	virtual	uchar			GetDateFormat( void ) const;
FBL_EXP	virtual	char			GetDateSep	 ( void ) const;
FBL_EXP	virtual	char			GetTimeSep	 ( void ) const;

FBL_EXP	virtual	void			SetDateFormat( uchar inValue );
FBL_EXP	virtual	void			SetDateSep	 (  char inValue );
FBL_EXP	virtual	void			SetTimeSep	 (  char inValue );


	public:///////////////////////////////////////////////////////////////////////////////
			
		// THE NEXT Methods ARE ALMOST INTERNAL:
		
			// Methods to work withstatic structure:

								// While the database structure isstatic and it is defined
								// by custom C++ VDK classes then we 
FBL_EXP	void					Create_OBL_BaseObject( VDK_BaseObject* inBaseObject );

			// Methods to work with dynamic structure:

								// OBL-objects was created by kernel on "Open database",
								// now we need to make VDK-objects using OBL-objects.
FBL_EXP	void			 		Create_VDK_BaseObjects( void );

								// Creates one VDK_BaseObject using internal object.
FBL_EXP	VDK_BaseObject*			Create_VDK_BaseObject( OBL_BaseObject* inBaseObject );

								// Returns VDK BaseObject connected to the inObject.
FBL_EXP	VDK_BaseObject*			GetBaseObject( OBL_BaseObject* inObject ) const; 

		bool					IsInited( void ) const { return (mDataBaseImp != NULL); }

FBL_EXP virtual	VDK_BaseObject*	NewVDKBaseObject( const char* inName );


		/* Methods for work with Links */
		
		void					AddLink		( VDK_Link& inLink );  
		void					RemoveLink	( VDK_Link& inLink ); 
		
								// Returns Link by name, case insensitive.
FBL_EXP	VDK_Link*				GetLink( const char* inName ) const;


		/* Methods to work with SQL and Cursors */
			
		void					AddCursor		( VDK_Cursor& inCursor );  
		void					RemoveCursor	( VDK_Cursor& inCursor ); 
		bool					Exists			( VDK_Cursor* inCursor );
									
		/* Internal: */

		OBL_DataBase*			GetImplementation( void ) const { return mDataBaseImp; }
FBL_EXP	void					DisposeSelf( void );


	protected:///////////////////////////////////////////////////////////////////////////////

		FBL_String				mName;
		FBL_String				mFullPath;
		
		OBL_DataBase*			mDataBaseImp;	
	
		ObjectsArray			mObjects;		// array of VDK_BaseObject.
		LinksArray				mLinks; 		// array of all links beetwen objects in domain,
												// realtions and pointers.

		CursorsArray			mCursors;


	public://///////////////////////////////////////////////////////////////////////////////////

		// These 2 data members allows easy to use VSDK to develop plugins
		// such as Valentina for REALbasic, Valentina for Java, which do have garbage collection.
			
			// An owner should set up this back reference, so when destructor of this class
			// will be called FROM kernel it will set back ref to NULL via mBackRef. 
			// The owner will be able to handle correctly this situation and won't try access
			// already destroyed kernel object.
		VDK_DataBase**			mBackRef;
				
			// here you can put pointer on your own additional data (or even structure)
			// if you need attach several variables to this C++ instance.	
		void*					mXtraData;	

			// Destructor ~VDK_DataBase() will call this pointer if you set it.
		void					(*mpFinalize)(VDK_DataBase* inDataBase );


	public:///////////////////////////////////////////////////////////////////////////////

		ushort					mUsersCount;	// Count of independent users/processes
												// Which use this database

	private:///////////////////////////////////////////////////////////////////////////////

		void 					Create_Common( void );
		void 					Open_Common( void );

		void 					Parse_AlterTable( 
									const char* inSqlCommand );
									
		void 					Parse_CreateTable( 
									const char* inSqlCommand );
									
		void 					Parse_DropTable( 
									const char* inSqlCommand );
		
		void 					Parse_DeleteStatement( 
									const char* inSqlCommand, 
									ulong*		outAffectedRows = NULL );
									
		void 					Parse_InsertStatement( 
									const char*	 	inSqlCommand, 
									ulong* 			outAffectedRows = NULL,
									BindParameters* inBindParameters = NULL );

		void 					Parse_InsertStatementValues(
									const char* inSqlCommand,
									ulong*		outAffectedRows,
									const char* tablename,
									FBL_Array<FBL_String*>& fieldNames,
									BindParameters* inBindParameters );

		void 					Parse_InsertStatementSelect(
									const char* inSqlCommand,
									ulong*		outAffectedRows,
									const char* tablename,
									FBL_Array<FBL_String*>& fieldNames );

		void 					Parse_UpdateStatement( 
									const char* 	inSqlCommand, 
									ulong*			outAffectedRows = NULL,
									BindParameters* inBindParameters = NULL );
 
		fieldInfo* 				Parse_CreateField( 
									char*& inSqlCommand );
		
		void 					Parse_execCreateField( 
									VDK_BaseObject* pBaseObject, 
									fieldInfo* pFieldInfo );

		void 					Parse_ConvertRDBLink( 
									const char* inSqlCommand );

		void 					Parse_Reindex( const char* inSqlCommand );
		void 					Parse_Compact( const char* inSqlCommand );
		void 					Parse_Defragment( const char* inSqlCommand );
		void 					Parse_Diagnose( const char* inSqlCommand );
	
}; 


/**********************************************************************************************/
inline VDK_BaseObject* VDK_DataBase::GetBaseObject( ulong inIndex ) const
{
	if( 1 <= inIndex && inIndex <= mObjects.GetCount() )
			return mObjects.GetItemAt(inIndex);
	else	return NULL;
}

/**********************************************************************************************/
inline void VDK_DataBase::AddLink( VDK_Link& inLink )  
{
	mLinks.AddItem( &inLink );
} 


/**********************************************************************************************/
inline void VDK_DataBase::RemoveLink( VDK_Link& inLink )  
{
	mLinks.RemoveItem( &inLink );
} 


/**********************************************************************************************/
inline void VDK_DataBase::AddCursor( VDK_Cursor& inCursor )  
{
	mCursors.AddItem( &inCursor );
} 


/**********************************************************************************************/
inline void VDK_DataBase::RemoveCursor( VDK_Cursor& inCursor )  
{
	mCursors.RemoveItem( &inCursor );
} 


/**********************************************************************************************/
inline bool VDK_DataBase::Exists( VDK_Cursor* inCursor )  
{
	return mCursors.FindIndex(inCursor) ? true : false;	
} 


/**********************************************************************************************/
inline VDK_DataBase* VDK_DataBase::GetDataBase( FBL_String inName )  
{
	if( sDBList )
	{
		ushort Count = (ushort) sDBList->GetCount();
		if( Count > 0 )
		{
			// we must check if this database already open ?
			for( ushort i = 1; i <= Count ; i++ )
			{
				VDK_DataBase* pDB = (VDK_DataBase*) sDBList->GetItemAt( i );
														
				if( FBL_String::CompareNoCase( pDB->GetName(), (char*) inName) == 0 )	
					 return pDB;
			} 
		}
	}
	
	// we have not found DataBase with this name.
	return NULL;
} 

inline bool dummy_ret_true( void )
{
	bool res = true;
	return res;
}
inline bool dummy_ret_false( void )
{
	bool res = false;
	return res;
}

/**********************************************************************************************/

#endif

