/**********************************************************************************************/
/* VDK_SearchTree.h	                                                					  	  */
/*                                                                       					  */
/* Valentina SDK    																		  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/

#ifndef __VDK_SearchTree_h 
	#define __VDK_SearchTree_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#	include <VSDK/FBL_String.h>
#else
#	include <FBL.h>
#	include <FBL_String.h>
#endif // VSDK_FRAMEWORK_STYLE


/**********************************************************************************************/
class 	FBL_String;

class 	VDK_Field;

class 	VDK_SearchTree;
class 	VDK_OR_Branch;
struct 	VDK_FieldCondition;

typedef	FBL_Array<VDK_FieldCondition*>	ConditionsArray;


/**********************************************************************************************/
class VDK_SearchTree	
{
	public://///////////////////////////////////////////////////////////////////////////////////
 
FBL_EXP							VDK_SearchTree( void );												
FBL_EXP virtual					~VDK_SearchTree( void );

FBL_EXP	void					Add_OR_Branch( VDK_OR_Branch* inBranch );
FBL_EXP	void					RemoveOR_Branch( VDK_OR_Branch* inBranch );			
		
		ushort					GetCount( void ) const { return mTotal; }
		
FBL_EXP	VDK_OR_Branch*			GetBranch( ushort inIndex ) const;
FBL_EXP	ushort					GetIndex( VDK_OR_Branch* inBranch ) const;
		
		bool					GetSearchSelectionFlag( void ) const { 
									return mSearchSelection; 	
								}
							
		void					SetSearchSelectionFlag( bool inFlag ) { 
									mSearchSelection = inFlag; 
								}				

	public://///////////////////////////////////////////////////////////////////////////////////

		VDK_OR_Branch*			mHeadPtr;
		VDK_OR_Branch*			mTailPtr;
		ushort					mTotal;

		bool					mSearchSelection;	// if this flag is true then search must be made 
													// in the current selection.
};


/**********************************************************************************************/
class VDK_OR_Branch
{
	friend class VDK_SearchTree;

	public://///////////////////////////////////////////////////////////////////////////////////
 
FBL_EXP							VDK_OR_Branch( void );
FBL_EXP							VDK_OR_Branch( const VDK_OR_Branch& inSrc );
 								
FBL_EXP	void					SetCondition( 	
 									VDK_Field* 	inField,
									const char*	inCondition );
 								
 		void					RemoveCondition( VDK_Field* inField ) {
 									ReplaceCondition(inField, NULL);
 								}
 														
   								// Remove condition for given field.
FBL_EXP	void					ReplaceCondition( VDK_Field* inField, VDK_Field* inNewField = NULL );


		/* Accessories: */

		ulong					GetCount( void )			
									{ return mConditions.GetCount(); }
		
		VDK_FieldCondition*		GetFieldCondition( ulong inIndex )
									{ return mConditions.GetItemAt(inIndex); }

		
		VDK_OR_Branch*			GetNextLink( void )			{ return mNextPtr;	}
		VDK_OR_Branch*			GetPrevLink( void )			{ return mPrevPtr;	}


	public://///////////////////////////////////////////////////////////////////////////////////

		VDK_OR_Branch*			mNextPtr;		// pointer to the next OR branch if any.
		VDK_OR_Branch*			mPrevPtr;		// pointer to the prev OR branch if any.
												
		ConditionsArray			mConditions;	// this is AND conditions for this OR branch.
												// F1 and F2 and F3

};


/**********************************************************************************************/
struct VDK_FieldCondition
{
		VDK_Field*				mField;			// pointer on field
		FBL_String				mCondition;		// its condition.

FBL_EXP							VDK_FieldCondition( VDK_Field* 	inField, 
													const char*	inCondition );
	
FBL_EXP							VDK_FieldCondition( const VDK_FieldCondition& inSrc );
};								


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __VDK_SearchTree_h

